/* Copyright (c) 2009 Christoph Studer <chstuder@gmail.com>
 * Copyright (c) 2010 Jan Berkel <jan.berkel@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.axet.smsgate.fragments;

import static android.content.Context.MODE_PRIVATE;
import static com.zegoggles.smssync.App.TAG;
import static com.zegoggles.smssync.mail.DataType.MMS;
import static com.zegoggles.smssync.mail.DataType.SMS;
import static com.zegoggles.smssync.preferences.Preferences.Keys.BACKUP_SETTINGS_SCREEN;
import static com.zegoggles.smssync.preferences.Preferences.Keys.ENABLE_AUTO_BACKUP;
import static com.zegoggles.smssync.preferences.Preferences.Keys.IMAP_SETTINGS;
import static com.zegoggles.smssync.preferences.Preferences.Keys.INCOMING_TIMEOUT_SECONDS;
import static com.zegoggles.smssync.preferences.Preferences.Keys.MAX_ITEMS_PER_SYNC;
import static com.zegoggles.smssync.preferences.Preferences.Keys.REGULAR_TIMEOUT_SECONDS;
import static com.zegoggles.smssync.preferences.Preferences.Keys.WIFI_ONLY;

import android.annotation.TargetApi;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.app.admin.DevicePolicyManager;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.provider.Telephony;
import android.support.v7.preference.CheckBoxPreference;
import android.support.v7.preference.ListPreference;
import android.support.v7.preference.Preference;
import android.support.v7.preference.PreferenceFragmentCompat;
import android.support.v7.preference.PreferenceScreen;
import android.text.TextUtils;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import com.github.axet.androidlibrary.preferences.AboutPreferenceCompat;
import com.github.axet.androidlibrary.preferences.AdminPreferenceCompat;
import com.github.axet.androidlibrary.preferences.OptimizationPreferenceCompat;
import com.github.axet.smsgate.R;
import com.github.axet.smsgate.activities.MainActivity;
import com.github.axet.smsgate.app.SMSApplication;
import com.github.axet.smsgate.services.NotificationListener;
import com.github.axet.smsgate.services.NotificationService;
import com.github.axet.smsgate.services.OnBootReceiver;
import com.github.axet.smsgate.services.ScheduleService;
import com.github.axet.smsgate.services.ImapSmsReplyService;
import com.github.axet.smsgate.widgets.ApplicationsPreference;
import com.github.axet.smsgate.widgets.DefaultSMSPreferenceCompat;
import com.github.axet.smsgate.widgets.IMAPSyncPreferenceCompat;
import com.github.axet.smsgate.widgets.NameFormatPreferenceCompat;
import com.github.axet.smsgate.widgets.RebootPreferenceCompat;
import com.zegoggles.smssync.App;
import com.zegoggles.smssync.Consts;
import com.zegoggles.smssync.activity.Dialogs;
import com.zegoggles.smssync.activity.auth.WebAuthActivity;
import com.zegoggles.smssync.mail.BackupImapStore;
import com.zegoggles.smssync.mail.DataType;
import com.zegoggles.smssync.preferences.AuthMode;
import com.zegoggles.smssync.preferences.AuthPreferences;
import com.zegoggles.smssync.preferences.BackupManagerWrapper;
import com.zegoggles.smssync.preferences.Preferences;
import com.zegoggles.smssync.service.ImapSmsService;
import com.zegoggles.smssync.tasks.RequestTokenTask;
import com.zegoggles.smssync.utils.AppLog;

import org.jetbrains.annotations.Nullable;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * This is the main activity showing the status of the SMS Sync service and
 * providing controls to configure it.
 */
public class SettingsFragment extends PreferenceFragmentCompat implements MainActivity.OnBackHandler, SharedPreferences.OnSharedPreferenceChangeListener {
    public static final int MIN_VERSION_MMS = Build.VERSION_CODES.ECLAIR;
    public static final int MIN_VERSION_BACKUP = Build.VERSION_CODES.FROYO;

    private static final int REQUEST_CHANGE_DEFAULT_SMS_PACKAGE = 1;
    private static final int REQUEST_PICK_ACCOUNT = 2;
    private static final int REQUEST_WEB_AUTH = 3;
    private static final int REQUEST_ADMIN = 4;

    enum Actions {
        Backup,
    }

    private Actions mActions;

    List<PreferenceScreen> back = new ArrayList<>();

    private AuthPreferences authPreferences;
    private Preferences preferences;
    Handler handler = new Handler();
    private
    @Nullable
    Uri mAuthorizeUri;
    BroadcastReceiver receiver;
    OptimizationPreferenceCompat.SettingsReceiver settings;

    private static Preference.OnPreferenceChangeListener sBindPreferenceSummaryToValueListener = new Preference.OnPreferenceChangeListener() {
        @Override
        public boolean onPreferenceChange(Preference preference, Object value) {
            String stringValue = value.toString();
            if (preference instanceof NameFormatPreferenceCompat) {
                preference.setSummary(((NameFormatPreferenceCompat) preference).getFormatted(stringValue));
            } else if (preference instanceof ListPreference) {
                // For list preferences, look up the correct display value in
                // the preference's 'entries' list.
                ListPreference listPreference = (ListPreference) preference;
                int index = listPreference.findIndexOfValue(stringValue);

                // Set the summary to reflect the new value.
                preference.setSummary(
                        index >= 0
                                ? listPreference.getEntries()[index]
                                : null);

            } else {
                preference.setSummary(stringValue);
            }
            return true;
        }
    };

    private static void bindPreferenceSummaryToValue(Preference preference) {
        // Set the listener to watch for value changes.
        preference.setOnPreferenceChangeListener(sBindPreferenceSummaryToValueListener);

        // Trigger the listener immediately with the preference's
        // current value.
        sBindPreferenceSummaryToValueListener.onPreferenceChange(preference,
                PreferenceManager.getDefaultSharedPreferences(preference.getContext()).getAll().get(preference.getKey()));
    }

    @Override
    public void onCreatePreferences(Bundle bundle, String s) {
        Context context = getActivity();

        authPreferences = new AuthPreferences(context);
        preferences = new Preferences(context);
        addPreferencesFromResource(R.xml.settings);

        if (!SMSApplication.firebaseEnabled(context)) {
            Preference pref = findPreference(SMSApplication.PREF_FIREBASESETTINGS);
            if (pref != null)
                pref.setVisible(false);
            Preference adm = findPreference(SMSApplication.PREF_ADMIN);
            if (pref != null)
                adm.setVisible(false);
        }

        int version = Build.VERSION.SDK_INT;
        if (version < MIN_VERSION_MMS) {
            CheckBoxPreference backupMms = (CheckBoxPreference) findPreference(MMS.backupEnabledPreference);
            backupMms.setEnabled(false);
            backupMms.setChecked(false);
            backupMms.setSummary(R.string.ui_backup_mms_not_supported);
        }
        if (preferences.shouldShowUpgradeMessage()) show(Dialogs.UPGRADE_FROM_SMSBACKUP);
        setPreferenceListeners(version >= MIN_VERSION_BACKUP);

        checkAndDisplayDroidWarning();

        preferences.migrateMarkAsRead();

        checkDefaultSmsApp();

        setupStrictMode();
        App.bus.register(this);

        SharedPreferences prefs = android.support.v7.preference.PreferenceManager.getDefaultSharedPreferences(context);
        prefs.registerOnSharedPreferenceChangeListener(this);

        receiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                String a = intent.getAction();
                if (a.equals(DevicePolicyManager.ACTION_DEVICE_OWNER_CHANGED)) {
                    ((AdminPreferenceCompat) findPreference(SMSApplication.PREF_ADMIN)).updateAdmin();
                }
            }
        };
        IntentFilter ff = new IntentFilter();
        ff.addAction(DevicePolicyManager.ACTION_DEVICE_OWNER_CHANGED);
        getActivity().registerReceiver(receiver, ff);

        settings = new OptimizationPreferenceCompat.SettingsReceiver(new Intent(context, ScheduleService.class), SMSApplication.PREF_OPTIMIZATION);
        settings.register(context);
    }

    @Override
    public void onResume() {
        super.onResume();
        Log.d(TAG, "onResume()");

        updateLastBackupTimes();
        updateAutoBackupSummary();
        updateAutoBackupEnabledSummary();
        updateImapFolderLabelFromPref();
        updateUsernameLabel(null);
        updateMaxItemsPerSync(null);

        updateImapSettings(!authPreferences.useXOAuth());

        updateNotifications();
        updateApps();

        OptimizationPreferenceCompat optimization = (OptimizationPreferenceCompat) findPreference(SMSApplication.PREF_OPTIMIZATION);
        optimization.enable(ScheduleService.class);
        optimization.onResume();

        RebootPreferenceCompat reboot = (RebootPreferenceCompat) findPreference(SMSApplication.PREF_REBOOT);
        reboot.setFragment(this);
        reboot.onResume();

        AdminPreferenceCompat admin = ((AdminPreferenceCompat) findPreference(SMSApplication.PREF_ADMIN));
        admin.setFragment(this, REQUEST_ADMIN);
        admin.setMessages(R.string.admin_web_description, AdminPreferenceCompat.ERASE_ALL_DATA, R.string.admin_web_erase, AdminPreferenceCompat.LOCK_SCREEN, R.string.admin_web_lock);
        admin.onResume();

        DefaultSMSPreferenceCompat sms = (DefaultSMSPreferenceCompat) findPreference(SMSApplication.PREF_DEFAULTSMS);
        sms.a = getActivity();
        sms.onResume();

        Preference delete = findPreference(SMSApplication.PREF_DELETE);
        bindPreferenceSummaryToValue(delete);
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();

        try {
            App.bus.unregister(this);
        } catch (Exception ignored) {
        }

        SharedPreferences prefs = android.support.v7.preference.PreferenceManager.getDefaultSharedPreferences(getActivity());
        prefs.unregisterOnSharedPreferenceChangeListener(this);

        if (receiver != null) {
            getActivity().unregisterReceiver(receiver);
            receiver = null;
        }
        settings.unregister(getContext());
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_about:
                show(Dialogs.ABOUT);
                return true;
            case R.id.menu_reset:
                show(Dialogs.RESET);
                return true;
            case R.id.menu_view_log:
                show(Dialogs.VIEW_LOG);

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    String getLastSyncText(final long lastSync) {
        return getString(R.string.status_idle_details,
                lastSync < 0 ? getString(R.string.status_idle_details_never) :
                        DateFormat.getDateTimeInstance().format(new Date(lastSync)));

    }

    private void updateLastBackupTimes() {
        for (DataType type : Arrays.asList(DataType.SMS)) {
            Preference pref = findPreference(type.backupEnabledPreference);
            if (pref != null)
                pref.setSummary(getLastSyncText(type.getMaxSyncedDate(getActivity())));
        }
    }

    private ConnectivityManager getConnectivityManager() {
        return (ConnectivityManager) getActivity().getSystemService(Context.CONNECTIVITY_SERVICE);
    }

    private void updateAutoBackupEnabledSummary() {
        Preference p = findPreference(ENABLE_AUTO_BACKUP.key);
        if (p == null) // sub screen
            return;
        p.setSummary(getEnabledBackupSummary());
        for (DataType t : Arrays.asList(DataType.SMS)) {
            addSummaryListener(new Runnable() {
                public void run() {
                    updateAutoBackupEnabledSummary();
                }
            }, t.backupEnabledPreference);
        }
    }

    protected String getEnabledBackupSummary() {
        final List<String> enabled = new ArrayList<String>();
        for (DataType dataType : DataType.enabled(getActivity())) {
            enabled.add(getString(dataType.resId));
        }
        StringBuilder summary = new StringBuilder();
        if (!enabled.isEmpty()) {
            summary.append(getString(R.string.ui_enable_auto_sync_summary, TextUtils.join(", ", enabled)));
            if (!getConnectivityManager().getBackgroundDataSetting()) {
                summary.append(' ').append(getString(R.string.ui_enable_auto_sync_bg_data));
            }
            if (preferences.isInstalledOnSDCard()) {
                summary.append(' ').append(getString(R.string.sd_card_disclaimer));
            }
        } else {
            summary.append(getString(R.string.ui_enable_auto_sync_no_enabled_summary));
        }
        return summary.toString();
    }

    private void updateAutoBackupSummary() {
        final Preference autoBackup = findPreference(BACKUP_SETTINGS_SCREEN.key);
        if (autoBackup != null) {
            final StringBuilder summary = new StringBuilder();

            final ListPreference regSchedule = (ListPreference)
                    findPreference(REGULAR_TIMEOUT_SECONDS.key);

            final ListPreference incomingSchedule = (ListPreference)
                    findPreference(INCOMING_TIMEOUT_SECONDS.key);

            summary.append(regSchedule.getTitle())
                    .append(": ")
                    .append(regSchedule.getEntry())
                    .append(", ")
                    .append(incomingSchedule.getTitle())
                    .append(": ")
                    .append(incomingSchedule.getEntry());

            if (preferences.isWifiOnly()) {
                summary.append(" (")
                        .append(findPreference(WIFI_ONLY.key).getTitle())
                        .append(")");
            }

            autoBackup.setSummary(summary.toString());
        }

        addSummaryListener(new Runnable() {
                               public void run() {
                                   updateAutoBackupSummary();
                               }
                           }, INCOMING_TIMEOUT_SECONDS.key,
                REGULAR_TIMEOUT_SECONDS.key,
                WIFI_ONLY.key);
    }

    private void addSummaryListener(final Runnable r, String... prefs) {
        for (String p : prefs) {
            Preference pref = findPreference(p);
            if (pref != null) {
                pref.setOnPreferenceChangeListener(
                        new Preference.OnPreferenceChangeListener() {
                            public boolean onPreferenceChange(Preference preference, final Object newValue) {
                                new Handler().post(new Runnable() {
                                    @Override
                                    public void run() {
                                        r.run();
                                        // TODO onContentChanged();
                                    }
                                });
                                return true;
                            }
                        });
            }
        }
    }

    private void updateUsernameLabel(String username) {
        if (username == null) {
            SharedPreferences prefs = getPreferenceManager().getSharedPreferences();
            username = prefs.getString(AuthPreferences.LOGIN_USER, getString(R.string.ui_login_label));
        }
        Preference pref = findPreference(AuthPreferences.LOGIN_USER);
        if (pref != null)
            pref.setTitle(username);
    }

    private void updateImapFolderLabelFromPref() {
        // String imapFolder = SMS.getFolder(getActivity(), NameFormatPreferenceCompat.DEMO);
        // Preference pref = findPreference(SMS.folderPreference);
        // pref.setTitle(imapFolder);
    }

    private void initiateBackup() {
        if (checkLoginInformation()) {
            if (preferences.isFirstBackup()) {
                show(Dialogs.FIRST_SYNC);
            } else {
                startBackup(false);
            }
        }
    }

    private boolean checkLoginInformation() {
        if (!authPreferences.isLoginInformationSet()) {
            show(Dialogs.MISSING_CREDENTIALS);
            return false;
        } else {
            return true;
        }
    }

    void performAction(Actions act) {
        performAction(act, preferences.confirmAction());
    }

    private void performAction(Actions act, boolean needConfirm) {
        if (needConfirm) {
            this.mActions = act;
            show(Dialogs.CONFIRM_ACTION);
        } else {
            if (Actions.Backup.equals(act)) {
                initiateBackup();
            }
        }
    }

    private void startBackup(boolean skip) {
        ImapSmsService.startManual(getActivity(), skip);
        ImapSmsReplyService.start(getActivity());
    }

    @TargetApi(Build.VERSION_CODES.KITKAT)
    private boolean isSmsBackupDefaultSmsApp() {
        Context context = getActivity();
        return Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT &&
                context.getPackageName().equals(Telephony.Sms.getDefaultSmsPackage(context));
    }

    protected void onPrepareDialog(int id, Dialog dialog) {
        switch (Dialogs.values()[id]) {
            case VIEW_LOG:
                View view = dialog.findViewById(AppLog.ID);
                if (view instanceof TextView) {
                    AppLog.readLog(App.LOG, (TextView) view);
                }
        }
    }

    protected Dialog onCreateDialog(final int id) {
        final Context context = getActivity();

        String title, msg;
        switch (Dialogs.values()[id]) {
            case MISSING_CREDENTIALS:
                title = getString(R.string.ui_dialog_missing_credentials_title);
                msg = authPreferences.useXOAuth() ?
                        getString(R.string.ui_dialog_missing_credentials_msg_xoauth) :
                        getString(R.string.ui_dialog_missing_credentials_msg_plain);
                break;
            case INVALID_IMAP_FOLDER:
                title = getString(R.string.ui_dialog_invalid_imap_folder_title);
                msg = getString(R.string.ui_dialog_invalid_imap_folder_msg);
                break;
            case FIRST_SYNC:
                DialogInterface.OnClickListener firstSyncListener =
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                startBackup(which == DialogInterface.BUTTON2);
                            }
                        };
                final int maxItems = preferences.getMaxItemsPerSync();
                final String syncMsg = maxItems < 0 ?
                        getString(R.string.ui_dialog_first_sync_msg) :
                        getString(R.string.ui_dialog_first_sync_msg_batched, maxItems);

                return new AlertDialog.Builder(context)
                        .setTitle(R.string.ui_dialog_first_sync_title)
                        .setMessage(syncMsg)
                        .setPositiveButton(R.string.ui_sync, firstSyncListener)
                        .setNegativeButton(R.string.ui_skip, firstSyncListener)
                        .create();
            case ABOUT:
                return AboutPreferenceCompat.buildDialog(context, R.raw.about).create();

            case VIEW_LOG:
                return AppLog.displayAsDialog(App.LOG, context);

            case RESET:
                return new AlertDialog.Builder(context)
                        .setTitle(R.string.ui_dialog_reset_title)
                        .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                reset();
                                dismissDialog(id);
                            }
                        })
                        .setMessage(R.string.ui_dialog_reset_message)
                        .setNegativeButton(android.R.string.cancel, null)
                        .create();

            case REQUEST_TOKEN:
                ProgressDialog req = new ProgressDialog(context);
                req.setTitle(null);
                req.setMessage(getString(R.string.ui_dialog_request_token_msg));
                req.setIndeterminate(true);
                req.setCancelable(false);
                return req;
            case ACCESS_TOKEN:
                ProgressDialog acc = new ProgressDialog(context);
                acc.setTitle(null);
                acc.setMessage(getString(R.string.ui_dialog_access_token_msg));
                acc.setIndeterminate(true);
                acc.setCancelable(false);
                return acc;
            case ACCESS_TOKEN_ERROR:
                title = getString(R.string.ui_dialog_access_token_error_title);
                msg = getString(R.string.ui_dialog_access_token_error_msg);
                break;
            case CONNECT:
                return new AlertDialog.Builder(context)
                        .setCustomTitle(null)
                        .setMessage(getString(R.string.ui_dialog_connect_msg, getString(R.string.app_name)))
                        .setNegativeButton(android.R.string.cancel, null)
                        .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                if (mAuthorizeUri != null) {
                                    startActivityForResult(new Intent(context, WebAuthActivity.class)
                                            .setData(mAuthorizeUri), REQUEST_WEB_AUTH);
                                }
                                dismissDialog(id);
                            }
                        }).create();
            case CONNECT_TOKEN_ERROR:
                return new AlertDialog.Builder(context)
                        .setCustomTitle(null)
                        .setMessage(R.string.ui_dialog_connect_token_error)
                        .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                            }
                        }).create();

            case ACCOUNTMANAGER_TOKEN_ERROR:
                return new AlertDialog.Builder(context)
                        .setCustomTitle(null)
                        .setMessage(R.string.ui_dialog_account_manager_token_error)
                        .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                handleFallbackAuth();
                            }
                        })
                        .setNegativeButton(android.R.string.no, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                            }
                        })
                        .create();
            case UPGRADE_FROM_SMSBACKUP:
                title = getString(R.string.ui_dialog_upgrade_title);
                msg = getString(R.string.ui_dialog_upgrade_msg);
                break;
            case BROKEN_DROIDX:
                title = getString(R.string.ui_dialog_brokendroidx_title);
                msg = getString(R.string.ui_dialog_brokendroidx_msg);
                break;
            case CONFIRM_ACTION:
                return new AlertDialog.Builder(context)
                        .setTitle(R.string.ui_dialog_confirm_action_title)
                        .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                if (SettingsFragment.this.mActions != null) {
                                    performAction(SettingsFragment.this.mActions, false);
                                }
                            }
                        })
                        .setMessage(R.string.ui_dialog_confirm_action_msg)
                        .setNegativeButton(android.R.string.cancel, null)
                        .create();
            case SMS_DEFAULT_PACKAGE_CHANGE:
                return new AlertDialog.Builder(context)
                        .setTitle(R.string.ui_dialog_sms_default_package_change_title)
                        .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                requestDefaultSmsPackageChange();
                            }
                        })
                        .setMessage(R.string.ui_dialog_sms_default_package_change_msg)
                        .create();
            default:
                return null;
        }
        return createMessageDialog(id, title, msg);
    }

    private void reset() {
        DataType.clearLastSyncData(getActivity());
        preferences.reset();
    }

    private Dialog createMessageDialog(final int id, String title, String msg) {
        return new AlertDialog.Builder(getActivity())
                .setTitle(title)
                .setMessage(msg)
                .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        dialog.cancel();
                    }
                })
                .create();
    }

    private void updateMaxItemsPerSync(String newValue) {
        updateMaxItems(MAX_ITEMS_PER_SYNC.key, preferences.getMaxItemsPerSync(), newValue);
    }

    private void updateMaxItems(String prefKey, int currentValue, String newValue) {
        Preference pref = findPreference(prefKey);
        if (pref == null)
            return;
        if (newValue == null) {
            newValue = String.valueOf(currentValue);
        }
        // XXX
        pref.setTitle("-1".equals(newValue) ? getString(R.string.all_messages) : newValue);
    }

    public CheckBoxPreference updateNotifications() {
        CheckBoxPreference notifications = (CheckBoxPreference) findPreference(SMSApplication.PREF_NOTIFICATION_LISTENER);
        if (notifications == null)
            return null;
        if (Build.VERSION.SDK_INT < 18) {
            if (notifications.isChecked())
                notifications.setChecked(NotificationService.enabled(getActivity()));
        } else {
            if (notifications.isChecked())
                notifications.setChecked(NotificationService.enabled(getActivity()) || NotificationListener.enabled(getActivity()));
        }
        return notifications;
    }

    public ApplicationsPreference updateApps() {
        ApplicationsPreference notifications = (ApplicationsPreference) findPreference(SMSApplication.PREF_APPS);
        return notifications;
    }

    private void show(Dialogs d) {
        showDialog(d.ordinal());
    }

    public SharedPreferences getPreferences(int i) {
        return PreferenceManager.getDefaultSharedPreferences(getActivity());
    }

    public void runOnUiThread(Runnable run) {
        handler.post(run);
    }

    Map<Integer, Dialog> dialogs = new HashMap<>();

    public void showDialog(int id) {
        Dialog d = onCreateDialog(id);
        onPrepareDialog(id, d);
        d.show();
        dialogs.put(id, d);
    }

    public void dismissDialog(int id) {
        Dialog d = dialogs.get(id);
        if (d != null)
            d.dismiss();
        dialogs.remove(id);
    }

    private void dismiss(Dialogs d) {
        try {
            dismissDialog(d.ordinal());
        } catch (IllegalArgumentException e) {
            // ignore
        }
    }

    private void updateImapSettings(boolean enabled) {
        Preference pref = findPreference(IMAP_SETTINGS.key);
        if (pref != null)
            pref.setEnabled(enabled);
    }

    private void setPreferenceListeners(boolean backup) {
        if (backup) {
            final Context context = getActivity();
            PreferenceManager.getDefaultSharedPreferences(context).registerOnSharedPreferenceChangeListener(
                    new SharedPreferences.OnSharedPreferenceChangeListener() {
                        public void onSharedPreferenceChanged(SharedPreferences prefs, String key) {
                            BackupManagerWrapper.dataChanged(context);
                        }
                    }
            );
        }

        final Context context = getActivity();

        Preference pref = findPreference(ENABLE_AUTO_BACKUP.key);
        if (pref != null) {
            pref.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
                public boolean onPreferenceChange(Preference preference, Object newValue) {
                    boolean isEnabled = (Boolean) newValue;
                    final ComponentName componentName = new ComponentName(context,
                            OnBootReceiver.class);
                    context.getPackageManager().setComponentEnabledSetting(componentName,
                            isEnabled ? PackageManager.COMPONENT_ENABLED_STATE_ENABLED :
                                    PackageManager.COMPONENT_ENABLED_STATE_DISABLED,
                            PackageManager.DONT_KILL_APP);

                    if (!isEnabled) {
                        ImapSmsReplyService.stop(context);
                        ImapSmsService.stop(context);
                    }
                    return true;
                }
            });
        }

        pref = findPreference(AuthPreferences.SERVER_AUTHENTICATION);
        if (pref != null) {
            pref.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
                public boolean onPreferenceChange(Preference preference, Object newValue) {
                    final boolean plain = (AuthMode.PLAIN) ==
                            AuthMode.valueOf(newValue.toString().toUpperCase(Locale.ENGLISH));
                    updateImapSettings(plain);
                    return true;
                }
            });
        }

        pref = findPreference(MAX_ITEMS_PER_SYNC.key);
        if (pref != null) {
            pref.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
                public boolean onPreferenceChange(Preference preference, Object newValue) {
                    updateMaxItemsPerSync(newValue.toString());
                    return true;
                }
            });
        }

        pref = findPreference(AuthPreferences.LOGIN_USER);
        if (pref != null) {
            pref.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
                public boolean onPreferenceChange(Preference preference, Object newValue) {
                    updateUsernameLabel(newValue.toString());
                    return true;
                }
            });
        }

        pref = findPreference(AuthPreferences.LOGIN_PASSWORD);
        if (pref != null) {
            pref.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
                public boolean onPreferenceChange(Preference preference, Object newValue) {
                    authPreferences.setImapPassword(newValue.toString());
                    return true;
                }
            });
        }

        pref = updateNotifications();
        if (pref != null) {
            pref.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
                @Override
                public boolean onPreferenceChange(Preference preference, Object o) {
                    if ((boolean) o == true) {
                        if (Build.VERSION.SDK_INT >= 18) {
                            if (!NotificationListener.enabled(getActivity())) {
                                NotificationListener.show(getActivity());
                            }
                        } else {
                            if (!NotificationService.enabled(getActivity())) {
                                NotificationService.show(getActivity());
                            }
                        }
                    }
                    return true;
                }
            });
        }

        updateApps();

        findPreference(SMS.folderPreference)
                .setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
                    public boolean onPreferenceChange(Preference preference, final Object newValue) {
                        String imapFolder = newValue.toString();
                        if (BackupImapStore.isValidImapFolder(imapFolder)) {
                            // preference.setTitle(imapFolder);
                            return true;
                        } else {
                            runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    show(Dialogs.INVALID_IMAP_FOLDER);
                                }
                            });
                            return false;
                        }
                    }
                });

        NameFormatPreferenceCompat.onResume(this);
        IMAPSyncPreferenceCompat.onResume(this);
    }

    @TargetApi(11)
    @SuppressWarnings({"ConstantConditions", "PointlessBooleanExpression"})
    private void setupStrictMode() {
    }

    private void checkAndDisplayDroidWarning() {
        if ("DROIDX".equals(Build.MODEL) ||
                "DROID2".equals(Build.MODEL) &&
                        Build.VERSION.SDK_INT == Build.VERSION_CODES.FROYO &&
                        !getPreferences(MODE_PRIVATE).getBoolean("droidx_warning_displayed", false)) {
            getPreferences(MODE_PRIVATE).edit().putBoolean("droidx_warning_displayed", true).commit();
            show(Dialogs.BROKEN_DROIDX);
        }
    }

    @TargetApi(Build.VERSION_CODES.KITKAT)
    private void requestDefaultSmsPackageChange() {
        final Intent changeIntent = new Intent(Telephony.Sms.Intents.ACTION_CHANGE_DEFAULT)
                .putExtra(Telephony.Sms.Intents.EXTRA_PACKAGE_NAME, getActivity().getPackageName());

        startActivityForResult(changeIntent, REQUEST_CHANGE_DEFAULT_SMS_PACKAGE);
    }

    @TargetApi(Build.VERSION_CODES.KITKAT)
    private void restoreDefaultSmsProvider(String smsPackage) {
        Log.d(TAG, "restoring SMS provider " + smsPackage);
        if (!TextUtils.isEmpty(smsPackage)) {
            final Intent changeDefaultIntent = new Intent(Telephony.Sms.Intents.ACTION_CHANGE_DEFAULT)
                    .putExtra(Telephony.Sms.Intents.EXTRA_PACKAGE_NAME, smsPackage);
            startActivity(changeDefaultIntent);
        }
    }

    private void handleFallbackAuth() {
        show(Dialogs.REQUEST_TOKEN);
        new RequestTokenTask(getActivity()).execute(Consts.CALLBACK_URL);
    }

    private void checkDefaultSmsApp() {
        if (isSmsBackupDefaultSmsApp()) {
            restoreDefaultSmsProvider(preferences.getSmsDefaultPackage());
        }
    }

    @Override
    public void onNavigateToScreen(PreferenceScreen preferenceScreen) {
        super.onNavigateToScreen(preferenceScreen);
    }

    @Override
    public boolean onPreferenceTreeClick(Preference preference) {
        if (preference instanceof PreferenceScreen) {
            back.add(getPreferenceScreen());
            ((MainActivity) getActivity()).onBackHandler(this);
            setPreferenceScreen((PreferenceScreen) preference);
            return true;
        }
        return super.onPreferenceTreeClick(preference);
    }

    @Override
    public void onBackPressed() {
        int last = back.size() - 1;
        PreferenceScreen b = back.get(last);
        setPreferenceScreen(b);
        back.remove(last);
        if (back.size() == 0)
            ((MainActivity) getActivity()).onBackHandler(null);
        else
            ((MainActivity) getActivity()).onBackHandler(this);
    }

    @Override
    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
        if (key.equals(SMSApplication.PREF_REBOOT)) {
            ((RebootPreferenceCompat) findPreference(SMSApplication.PREF_REBOOT)).updateReboot();
        }
        if (key.equals(SMSApplication.PREF_WIFIRESTART))
            ScheduleService.startIfEnabled(getContext());
    }

    public Preference findPreference(CharSequence key) {
        if (back.isEmpty()) {
            return super.findPreference(key);
        } else {
            return back.get(0).findPreference(key);
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == REQUEST_ADMIN) {
            ((AdminPreferenceCompat) findPreference(SMSApplication.PREF_ADMIN)).onActivityResult(resultCode, data);
        }
    }

    @Override
    public void onDisplayPreferenceDialog(Preference preference) {
        if (preference instanceof NameFormatPreferenceCompat) {
            NameFormatPreferenceCompat.show(this, preference.getKey());
            return;
        }
        super.onDisplayPreferenceDialog(preference);
    }
}
